from django.core.paginator import Paginator
from django.utils.translation import ugettext as _
from django.db.models import Q
from django.db import connections
from django.db.models import Count

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from jmb.core.utils.functions import import_class
from jmb.core.utils.cmsutils import get_abs_url_from_slug
from jmb.news.models import News, Category
from jmb.news.cms.models import NewsSettings, NewsSearchSettings, CategorySettings

class NewsPlugin(CMSPluginBase):
    model = NewsSettings
    name = _("News List")
    module = _("News")
    admin_preview = False

    def render(self, context, instance, placeholder):
        page = 1
        self.render_template = instance.template_name
        ordering = instance.ordering.lower().replace(" ","").split(",")
        objects = News.objects.filter(status=1)
        query_string = None

        # raggruppa per data
        if instance.date_group:
            # the following will return an aggregate struct like this (e.g. group by month):
            # [{'news_count': 1, 'month': datetime.datetime(2013, 10, 1, 0, 0)}, ..., ...]
            aggregates_list = News.objects.extra(
                select={
                    instance.date_group: connections[News.objects.db].ops.date_trunc_sql(
                        instance.date_group, 'event_date'
                    )
                },
            ).values(instance.date_group).annotate(
                news_count=Count('event_date')
            )
            # ordering ('-event_date')
            aggregates_list = sorted(aggregates_list, key=lambda x: x[instance.date_group], reverse=True)

            context.update({
                'dates_aggregate': aggregates_list
            })
        else:
            #inclusioni
            if instance.slugs_included:
                slugs = instance.slugs_included.lower().replace(" ","").split(",")
                objects = objects.filter(slug__in = slugs)
            if instance.category_slugs_included:
                category_slugs = instance.category_slugs_included.lower().replace(" ","").split(",")
                objects = objects.filter(categories__slug__in = category_slugs)

            # esclusioni
            if instance.slugs_excluded:
                exclusion_slugs = instance.slugs_excluded.lower().replace(" ","").split(",")
                objects = objects.exclude(slug__in = exclusion_slugs)
            if instance.category_slugs_excluded:
                exclusion_category_slugs = instance.category_slugs_excluded.lower().replace(" ","").split(",")
                objects = objects.exclude(categories__slug__in = exclusion_category_slugs)

            # filtro date
            if instance.pub_date_begin:
                objects = objects.filter(event_date__gte= instance.pub_date_begin)
            if instance.pub_date_end:
                objects = objects.filter(event_date__lte= instance.pub_date_end)

            request = context['request']
            par_get = {}
            if request.GET:

                for key in request.GET.keys():
                    if request.GET.get(key) != "":
                        par_get[key] = request.GET.get(key)
                        
                # pagina
                if "page" in par_get and par_get['obj_page'] == 'news':
                    page = par_get['page']

                if 'obj' in par_get and par_get['obj'] == 'news':
                    query_dict = {}
                    query_list = []
                    for term in request.META['QUERY_STRING'].split("&"):
                        try:
                            query_dict[term.split("=")[0]] = term.split("=")[1]
                        except IndexError:
                            continue
                    for key, value in query_dict.items():
                        if key != 'page' and key != 'obj_page' and value != '':
                            query_list.append(key+"="+value)
                    query_string = "&".join(query_list)

                # mese
                    if 'month' in par_get:
                        month = par_get['month']
                        par_get.pop('month')
                    else:
                        month = None

                    if "page" in par_get and par_get['obj_page'] == 'news':
                        page = par_get['page']

                    # cancellazione per evitare interferenze con altre ricerche
                    if "page" in par_get:
                        del par_get['page']

                    # cancellazione per evitare interferenze con altre ricerche
                    if "obj_page" in par_get:
                        del par_get['obj_page']

                    if "obj" in par_get and par_get['obj'] == 'news':
                        par_get.pop('obj')

                    # inclusione parametri titolo in ricerca su titolo e descrizione
                    if "title__icontains" in par_get:
                        if "full_description__icontains" in par_get:
                            par_get["full_description__icontains"] += par_get["title__icontains"]
                        else:
                             par_get["full_description__icontains"] = par_get["title__icontains"]


                    try:
                        # se presente solo titolo, cerca termini titolo in campi 'title' e 'full_description'
                        if 'title__icontains' in par_get:
                            title = par_get.pop('title__icontains')
                            full_description = par_get.pop('full_description__icontains')
                            objects = objects.filter(
                                Q(title__icontains=title)|Q(full_description__icontains=full_description),
                                **par_get
                            )
                        else:
                            objects = objects.filter(**par_get)
                        if month:
                            month = int(month)
                            if month in range(1,13):
                                objects = objects.filter(event_date__month=month)
                    except Exception, e:
                        pass

        if instance.news_page_slug:
            news_page_slug = get_abs_url_from_slug(instance.news_page_slug)
            news_page_slug = news_page_slug[:-1] if news_page_slug[-1] == "/" else news_page_slug
            context.update({
                'news_page_slug':news_page_slug
            })

        objects = objects.order_by(*ordering)


        paginator = Paginator(objects, instance.objects_per_page)
        if not query_string:
            query_string = ""
        objects = paginator.page(page)
        context.update({
            'objects': objects,
            'instance': instance,
            'placeholder': placeholder,
            'query_string': query_string
            })
        return context


plugin_pool.register_plugin(NewsPlugin)

class NewsSearchPlugin(CMSPluginBase):

    model = NewsSearchSettings
    name = _("News Search")
    module = _("News")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        self.render_template = instance.template_name
        form_class = import_class(instance.form)
        try:
            if request.GET:
                form = form_class(request.GET)
            else:
                form = form_class()
        except:
            form = form_class()
        context.update({
            'form':form,
            'instance': instance,
            'placeholder': placeholder,
        })
        return context

plugin_pool.register_plugin(NewsSearchPlugin)

class CategoryPlugin(CMSPluginBase):
    model = CategorySettings
    name = _("Category List")
    module = _("News")
    admin_preview = False

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        ordering = instance.ordering.lower().replace(" ","").split(",")
        objects = Category.objects.filter(status=1)

        #inclusioni
        if instance.slugs_included:
            slugs = instance.slugs_included.lower().replace(" ","").split(",")
            objects = objects.filter(slug__in = slugs)

        # esclusioni
        if instance.slugs_excluded:
            exclusion_slugs = instance.slugs_excluded.lower().replace(" ","").split(",")
            objects = objects.exclude(slug__in = exclusion_slugs)

        request = context['request']
        par_get = {}
        query_string = None
        if request.GET:
            for key in request.GET.keys():
                if request.GET.get(key) != "":
                    par_get[key] = request.GET.get(key)
            if 'obj' in par_get and par_get['obj'] == 'news':
                query_dict = {}
                query_list = []
                for term in request.META['QUERY_STRING'].split("&"):
                    query_dict[term.split("=")[0]] = term.split("=")[1]
                for key, value in query_dict.items():
                    if (
                        key != 'page' and key != 'obj_page' and value != ''
                        # elimina il filtro per categoria nel query_string
                        and key != 'categories__slug'
                    ):
                        query_list.append(key+"="+value)
                query_string = "&".join(query_list)

            if "obj" in par_get and par_get['obj'] == 'news':
                par_get.pop('obj')

        if instance.news_page_slug:
            news_page_slug = get_abs_url_from_slug(instance.news_page_slug)
            news_page_slug = news_page_slug[:-1] if news_page_slug[-1] == "/" else news_page_slug
            context.update({
                'news_page_slug':news_page_slug
            })

        objects = objects.order_by(*ordering)

        context.update({
            'objects': objects,
            'instance': instance,
            'placeholder': placeholder,
            'query_string': query_string
        })
        return context

plugin_pool.register_plugin(CategoryPlugin)